@@ -7,16 +7,16 @@ from __future__ import print_function
 
 import os
 import sys
+import traceback
 
 from jupyter_core.application import JupyterApp, base_flags, base_aliases
 
 from traitlets import Bool, Unicode
 
-from ._version import __version__
 from .commands import (
     install_extension, uninstall_extension, list_extensions,
     enable_extension, disable_extension,
-    link_package, unlink_package, build, _get_linked_packages
+    link_package, unlink_package, build, get_app_version
 )
 
 
@@ -25,13 +25,19 @@ flags['no-build'] = (
     {'BaseExtensionApp': {'should_build': False}},
     "Defer building the app after the action."
 )
+flags['clean'] = (
+    {'BaseExtensionApp': {'should_clean': True}},
+    "Cleanup intermediate files after the action."
+)
 
 aliases = dict(base_aliases)
 aliases['app-dir'] = 'BaseExtensionApp.app_dir'
 
+VERSION = get_app_version()
+
 
 class BaseExtensionApp(JupyterApp):
-    version = __version__
+    version = VERSION
     flags = flags
     aliases = aliases
 
@@ -41,6 +47,23 @@ class BaseExtensionApp(JupyterApp):
     should_build = Bool(False, config=True,
         help="Whether to build the app after the action")
 
+    should_clean = Bool(False, config=True,
+        help="Whether temporary files should be cleaned up after building jupyterlab")
+
+    def start(self):
+        try:
+            self.run_task()
+        except Exception as ex:
+            _, _, exc_traceback = sys.exc_info()
+            msg = traceback.format_exception(ex.__class__, ex, exc_traceback)
+            for line in msg:
+                self.log.debug(line)
+            self.log.error(str(ex))
+            sys.exit(1)
+
+    def run_task(self):
+        pass
+
     def _log_format_default(self):
         """A default format for messages"""
         return "%(message)s"
@@ -51,55 +74,49 @@ class InstallLabExtensionApp(BaseExtensionApp):
     should_build = Bool(True, config=True,
         help="Whether to build the app after the action")
 
-    def start(self):
+    def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
-        [install_extension(arg, self.app_dir, logger=self.log)
-         for arg in self.extra_args]
+        [install_extension(arg, self.app_dir, logger=self.log) for
+         arg in self.extra_args]
+
         if self.should_build:
-            try:
-                build(self.app_dir, logger=self.log)
-            except Exception as e:
-                for arg in self.extra_args:
-                    uninstall_extension(arg, self.app_dir, logger=self.log)
-                raise e
+            build(self.app_dir, clean_staging=self.should_clean,
+                 logger=self.log)
 
 
 class LinkLabExtensionApp(BaseExtensionApp):
     description = """
-    Link labextension(s) or packages.
+    Link local npm packages that are not lab extensions.
 
-    Links a package to the JupyterLab build process.  If the package is
-    an extension, it will also be installed as an extension.  A linked
+    Links a package to the JupyterLab build process. A linked
     package is manually re-installed from its source location when
     `jupyter lab build` is run.
     """
     should_build = Bool(True, config=True,
         help="Whether to build the app after the action")
 
-    def start(self):
+    def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
         [link_package(arg, self.app_dir, logger=self.log)
          for arg in self.extra_args]
+
         if self.should_build:
-            try:
-                build(self.app_dir, logger=self.log)
-            except Exception as e:
-                for arg in self.extra_args:
-                    unlink_package(arg, self.app_dir, logger=self.log)
-                raise e
+            build(self.app_dir, clean_staging=self.should_clean,
+                  logger=self.log)
 
 
 class UnlinkLabExtensionApp(BaseExtensionApp):
-    description = "Unlink labextension(s) or packages by name or path"
+    description = "Unlink packages by name or path"
     should_build = Bool(True, config=True,
         help="Whether to build the app after the action")
 
-    def start(self):
+    def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
         ans = any([unlink_package(arg, self.app_dir, logger=self.log)
                    for arg in self.extra_args])
         if ans and self.should_build:
-            build(self.app_dir, logger=self.log)
+            build(self.app_dir, clean_staging=self.should_clean,
+                  logger=self.log)
 
 
 class UninstallLabExtensionApp(BaseExtensionApp):
@@ -107,25 +124,26 @@ class UninstallLabExtensionApp(BaseExtensionApp):
     should_build = Bool(True, config=True,
         help="Whether to build the app after the action")
 
-    def start(self):
+    def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
         ans = any([uninstall_extension(arg, self.app_dir, logger=self.log)
                    for arg in self.extra_args])
         if ans and self.should_build:
-            build(self.app_dir, logger=self.log)
+            build(self.app_dir, clean_staging=self.should_clean,
+                  logger=self.log)
 
 
 class ListLabExtensionsApp(BaseExtensionApp):
     description = "List the installed labextensions"
 
-    def start(self):
+    def run_task(self):
         list_extensions(self.app_dir, logger=self.log)
 
 
 class EnableLabExtensionsApp(BaseExtensionApp):
     description = "Enable labextension(s) by name"
 
-    def start(self):
+    def run_task(self):
         [enable_extension(arg, self.app_dir, logger=self.log)
          for arg in self.extra_args]
 
@@ -133,7 +151,7 @@ class EnableLabExtensionsApp(BaseExtensionApp):
 class DisableLabExtensionsApp(BaseExtensionApp):
     description = "Disable labextension(s) by name"
 
-    def start(self):
+    def run_task(self):
         [disable_extension(arg, self.app_dir, logger=self.log)
          for arg in self.extra_args]
 
@@ -148,7 +166,7 @@ jupyter labextension uninstall <extension name>  # uninstall a labextension
 class LabExtensionApp(JupyterApp):
     """Base jupyter labextension command entry point"""
     name = "jupyter labextension"
-    version = __version__
+    version = VERSION
     description = "Work with JupyterLab extensions"
     examples = _examples
 
@@ -159,7 +177,7 @@ class LabExtensionApp(JupyterApp):
         link=(LinkLabExtensionApp, "Link labextension(s)"),
         unlink=(UnlinkLabExtensionApp, "Unlink labextension(s)"),
         enable=(EnableLabExtensionsApp, "Enable labextension(s)"),
-        disable=(DisableLabExtensionsApp, "Disable labextensions(s)")
+        disable=(DisableLabExtensionsApp, "Disable labextension(s)"),
     )
 
     def start(self):
